unit MultiControlUnit;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls, ComCtrls;

type
  TMultiControl = class(TForm)
    RadioGroup1: TRadioGroup;
    RadioButton1: TRadioButton;
    RadioButton2: TRadioButton;
    RadioButton3: TRadioButton;
    TrackBar1: TTrackBar;
    Button1: TButton;
    Label1: TLabel;
    Label2: TLabel;
    procedure initprogram(Sender: TObject);
    procedure RadioButton2Click(Sender: TObject);
    procedure RadioButton3Click(Sender: TObject);
    procedure RadioButton1Click(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure TrackBar1Change(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    procedure idleloop(Sender: TObject; var Done: boolean);
  end;

var
  MultiControl: TMultiControl;

implementation

{$R *.dfm}

const maxmax = 501;

type coefftype = array[1..maxmax, 1..maxmax] of single;
     vectortype = array[1..maxmax] of single;

var ch: char;
    qi, p, e, o, r, temp, d: vectortype;
    inputCoeffs,w: coefftype;
    oldr, oldp, oldq: array[1..maxmax] of integer;
    i,j,maxy, maxx, stage: integer;
    count: longint;
    dt,ko, sumerr, t: double;
    s: string;
    maxmatrix: integer;

{v2 = M*v1 premultiply vector by square matrix}
procedure MV(var M:coefftype; var v1, v2: vectortype);
var i,j: integer;
    result: vectortype;
begin
 for i := 1 to maxmatrix do
 begin
  v2[i] := 0.0;
  for j := 1 to  maxmatrix do
    v2[i] := v2[i] + M[i,j]*v1[j]
 end;
end;

procedure VS(var a,b,c: vectortype); { C = A - B Vector subtract}
var i: integer;
begin
 for i := 1 to  maxmatrix do
 c[i] := a[i] - b[i];
 if c[i] > 100 then c[i] := 100
 else
 if c[i] < -100 then c[i] := -100;
end;

procedure VA(var a,b,c: vectortype);  { C = A + B vector add}
var i: integer;
begin
 for i := 1 to  maxmatrix do
 c[i] := a[i] + b[i];
end;

{B = k*int(A) vector integrate}
procedure VI(var a,b: vectortype; k: double);
var i: integer;
begin
 for i := 1 to  maxmatrix do
  b[i] := b[i] + k*a[i]*dt;
end;

procedure showmatrix(var m:coefftype; a,b: integer);
var maxj: integer;
begin
  if maxmatrix < 20 then maxj := maxmatrix else maxj := 20;
  for i := 1 to  maxmatrix do
   for j := 1 to  maxj do
   with multicontrol.canvas do
     begin
      textout(a + 4*(i-1), b - 1 + j,
        floattostrf(m[j,i],fffixed,4,1));
     end;
end;

procedure showvector(var v:vectortype; a,b: integer);
var maxi: integer;
begin
  if maxmatrix < 15 then maxi := maxmatrix else maxi := 15;
  for i := 1 to  maxi do
   with multicontrol.canvas do
     begin
      textout(a + 4*(i-1), b +i,
        floattostrf(v[i],fffixed,4,1));
     end;
end;

procedure controlloops;
begin
 MV(inputCoeffs,qi, p);    { p = coeffs*qi}
 VS(r,p,e);                { e = r - p}
 VI(e,o,ko);               { o = o + ko*int(e)}
 MV(w,o,temp);             { temp = w^*o}
 VA(temp, d, qi);          { qi = temp + d}
end;

procedure showvariables;
var i,j: integer;
    s: double;
begin
 s := (maxx - 15)/(maxmatrix - 1);
 for i := 1 to maxmatrix do
 with multicontrol,canvas do
 begin
  j := round(s*(i-1)) + 3;
  pen.Color := clBlack;
  ellipse(j-3,maxy div 2 - oldr[i] - 3,j+3,maxy div 2 - oldr[i] + 3);
  ellipse(j-1,maxy div 2 - oldp[i] - 1,j+1,maxy div 2 - oldp[i] + 1);
  ellipse(j-1,maxy div 2 - oldq[i] - 1,j+1,maxy div 2 - oldq[i] + 1);
  oldr[i] := round(r[i] /5.0);
  oldp[i] := round(p[i] /5.0);
  oldq[i] := round(qi[i] /5.0);
  pen.color := RGB(200,200,200);
  ellipse(j-3,maxy div 2 - oldr[i] - 3,j+3,maxy div 2 - oldr[i] + 3);
  pen.color := RGB(250,100,0);
  ellipse(j-1,maxy div 2 - oldp[i] - 1,j+1,maxy div 2 - oldp[i] + 1);
  pen.Color := RGB(150,0,255);
  ellipse(j-1,maxy div 2 - oldq[i] - 1,j+1,maxy div 2 - oldq[i] + 1);
  sumerr := sumerr + abs(e[i]);
 end;
end;

procedure iterate;
begin
 case stage of
  1:with multicontrol,canvas do
     begin
      brush.color := clBlack;
      fillrect(rect(0,26,clientwidth,clientheight));
      label1.Visible := false;
      trackbar1.visible := false;
      button1.font.Color := clRED;
      button1.caption := 'STOP';
      label2.visible := false;
      stage := 2;
     end;
  2: begin
      controlloops;
      showvariables;
      t := t + dt*2*pi/100.0*maxmatrix/500.0;
      if t > 2*pi then t := 0;
      r[maxmatrix div 2] := 1000.0*sin(t);
      r[maxmatrix div 4] := 1000.0*cos(t);
      r[3*maxmatrix div 4] := 1000.0*cos(t);
  end;
 end;
end;

procedure TMulticontrol.idleloop(Sender: TObject; var Done: boolean);
begin
 done := false;
 iterate;
end;

procedure TMultiControl.RadioButton1Click(Sender: TObject);
begin
 for i := 1 to maxmatrix do
         r[i] := 1000*sin(i*2*pi/maxmatrix);

end;

procedure TMultiControl.RadioButton2Click(Sender: TObject);
begin
 for i := 1 to maxmatrix do
         r[i] := -1000*cos(i*2*pi/maxmatrix);
end;

procedure TMultiControl.RadioButton3Click(Sender: TObject);
begin
for i := 1 to maxmatrix do
         r[i] := 2000*(random - 0.5);
end;

procedure initmatrix;
var i,j: integer;
    sum, rnum: double;
begin
 for i := 1 to  maxmatrix do
  for j := 1 to  maxmatrix do
   inputCoeffs[i,j] := 1.98*(random - 0.5);
 ch := chr(0);
 count := 0;
 sumerr := 100;
 for i := 1 to maxmatrix do
  begin
   o[i] := 1000*(random - 0.5);
   qi[i] := 3000*(random - 0.5);
   r[i] := 1500*(random - 0.5);
   p[i] := qi[i];
   d[i] := 0.0; // * (random - 0.5);
  end;
 t := 0.0;
 for i := 1 to  maxmatrix do
  for j := 1 to maxmatrix do
   w[j,i] := inputCoeffs[i,j]; {W = transpose(inputCoeffs)}
 ko := 0.03*50/maxmatrix;
 dt := 0.9;
end;

procedure TMultiControl.initprogram(Sender: TObject);
begin
 randomize;
 maxmatrix := 50;
 initmatrix;
 maxy := clientheight;
 maxx := clientwidth;
 application.onIdle := idleloop;
 stage := 0;
 end;

procedure TMultiControl.Button1Click(Sender: TObject);
begin
 if stage = 2 then application.terminate;
 stage := 1;
end;

procedure TMultiControl.TrackBar1Change(Sender: TObject);
var i: integer;
begin
 i := trackbar1.position;
 case i of
  1: maxmatrix := 3;
  2: maxmatrix := 5;
  3: maxmatrix := 10;
  4: maxmatrix := 30;
  5: maxmatrix := 50;
  6: maxmatrix := 100;
  7: maxmatrix := 300;
  8: maxmatrix := 500;
 end;
  label2.caption := 'Number Points = ' + inttostr(maxmatrix);
  initmatrix;
end;

end.
